﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Mail;
using System.Security.Cryptography;
using System.Text;
using System.Web;

namespace GestorDespeses.Utils
{

    //Aquesta clase ha estat extreta de https://gist.github.com/kristopherjohnson/3021045
    public static class SHA1Util
    {
        /// <summary>
        /// Compute hash for string encoded as UTF8
        /// </summary>
        /// <param name="s">String to be hashed</param>
        /// <returns>40-character hex string</returns>
        public static string SHA1HashStringForUTF8String(string s)
        {
            byte[] bytes = Encoding.UTF8.GetBytes(s);

            var sha1 = SHA1.Create();
            byte[] hashBytes = sha1.ComputeHash(bytes);
            return HexStringFromBytes(hashBytes);
        }

        /// <summary>
        /// Convert an array of bytes to a string of hex digits
        /// </summary>
        /// <param name="bytes">array of bytes</param>
        /// <returns>String of hex digits</returns>
        public static string HexStringFromBytes(byte[] bytes)
        {
            var sb = new StringBuilder();
            foreach (byte b in bytes)
            {
                var hex = b.ToString("x2");
                sb.Append(hex);
            }
            return sb.ToString();
        }
    }

    public static class DateUtils 
    {
        /// <summary>
        /// Comproba si un dia cau en cap de setmana
        /// </summary>
        /// <param name="date">Dia a comprobar si es cap de setmana</param>
        /// <returns>True si el dia es cap de setmana, False si no</returns>
        /// <exception cref="GestorDespeses.Model.Exceptions.GestorDespesesUnknownException">Error en el model desconegut.</exception>
        public static bool isDateWeekend(DateTime date)
        {
            bool result = false;
            if (date.Date.DayOfWeek.Equals(DayOfWeek.Sunday) ||
                date.Date.DayOfWeek.Equals(DayOfWeek.Saturday))
            {
                result = true;
            }
            return result;
        }
    }

    public static class RandomUtils
    {
        private static Random random = new Random();

        /// <summary>
        /// Genera una cadena aleatòria de longitud fixada per el paràmetre d'entrada
        /// </summary>
        /// <param name="lenght">Longitud de la cadena a generar</param>
        /// <returns>Cadena aleatòria generada</returns>
        public static string RandomString(int lenght)
        {
            var chars = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
            return new string(Enumerable.Repeat(chars, lenght).Select(s => s[random.Next(s.Length)]).ToArray());
        }
    }

    public static class SMTPUtils
    {
        /// <summary>
        /// Envía un correu electronic.
        /// </summary>
        /// <param name="fromEmail">Direcció de correu del remitent</param>
        /// <param name="fromFullName">Nom del remitent</param>
        /// <param name="password">Contrasenya del remitent per el host</param>
        /// <param name="host">Direcció del servidor de correu</param>
        /// <param name="port">Port del servidor de correu</param>
        /// <param name="enableSSL">True per habilitar SSL, False per enviaro en clar.</param>
        /// <param name="toEmail">Direcció de correu del destinatari</param>
        /// <param name="toFullName">Nom del destinatari</param>
        /// <param name="subject">Asumpte del correu</param>
        /// <param name="body">Cos del correu</param>
        public static void sendMail(string fromEmail,
            string fromFullName,
            string password,
            string host,
            int port,
            bool enableSSL,
            string toEmail, 
            string toFullName, 
            string subject, 
            string body)
        {
            var fromAddress = new MailAddress(fromEmail, fromFullName);
            var toAddress = new MailAddress(toEmail, toFullName);
            var smtp = new SmtpClient
            {
                Host = host, 
                Port = port,
                EnableSsl = enableSSL,
                DeliveryMethod = SmtpDeliveryMethod.Network,
                UseDefaultCredentials = false,
                Credentials = new NetworkCredential(fromAddress.Address, password)
            };
            using (var message = new MailMessage(fromAddress, toAddress) { Subject = subject, Body = body })
            { 
                smtp.Send(message); 
            }
        }

        /// <summary>
        /// Comproba si una cadena és una direcció de correu electronic vàlida.
        /// </summary>
        /// <param name="email">Direcció de correu a comprobar.</param>
        /// <returns>True si la direcció té el format correcte, False si no.</returns>
        public static bool IsValidEmail(string email)
        {
            try
            {
                var addr = new System.Net.Mail.MailAddress(email);
                return true;
            }
            catch
            {
                return false;
            }
        }
    }
}