﻿using System;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using GestorDespeses.BI;
using GestorDespeses.Model;
using GestorDespeses.Exceptions;

namespace GestorDespesesLibraryTest
{
    [TestClass]
    public class UsersTests
    {
        private static int generatedUser = -1;
        private static int user1Id = -1;
        private static int user2Id = -1;

        [TestMethod]
        public void createWorkerUser()
        {
            UserBI usersManager = new UserBI();
            int res = usersManager.addUser("xminana", 
                "password", 
                "xminana@uoc.edu", 
                "Javier Miñana Garcia", 
                UserType.WORKER);
            Assert.AreNotEqual(res, -1);
            generatedUser = res;
        }

        [TestMethod]
        public void validateUser()
        {
            UserBI usersManager = new UserBI();
            bool res = usersManager.validateUser("xminana","password");
            Assert.AreEqual(res, true);
        }

        [TestMethod]
        public void validateUserFailedPassword()
        {
            UserBI usersManager = new UserBI();
            bool res = usersManager.validateUser("xminana", "bad_password");
            Assert.AreEqual(res, false);
        }

        [TestMethod]
        public void validateUserFailedUserNotExists()
        {
            UserBI usersManager = new UserBI();
            bool res = usersManager.validateUser("unknown", "password");
            Assert.AreEqual(res, false);
        }

        [TestMethod]
        public void getUserById()
        {
            UserBI usersManager = new UserBI();
            User res = usersManager.getUser(generatedUser);
            Assert.AreEqual(res.UserId, generatedUser);
        }

        [TestMethod]
        [ExpectedException(typeof(GestorDespesesException))]
        public void getUserByIdNotExists()
        {
            UserBI usersManager = new UserBI();
            User res = usersManager.getUser(999999);
        }

        [TestMethod]
        public void getUserByUsername()
        {
            UserBI usersManager = new UserBI();
            User res = usersManager.getUserByUsername("xminana");
            Assert.AreEqual(res.UserId, generatedUser);
        }

        [TestMethod]
        [ExpectedException(typeof(GestorDespesesException))]
        public void getUserByUsernameNotExists()
        {
            UserBI usersManager = new UserBI();
            User res = usersManager.getUserByUsername("unknown");
        }

        [TestMethod]
        public void getUserByEmail()
        {
            UserBI usersManager = new UserBI();
            User res = usersManager.getUserByEmail("xminana@uoc.edu");
            Assert.AreEqual(res.UserId, generatedUser);
        }

        [TestMethod]
        [ExpectedException(typeof(GestorDespesesException))]
        public void getUserByNotExists()
        {
            UserBI usersManager = new UserBI();
            User res = usersManager.getUserByEmail("unknown@uoc.edu");
        }

        [TestMethod]
        [ExpectedException(typeof(GestorDespesesException))]
        public void createDuplicatedUsernameUser()
        {
            UserBI usersManager = new UserBI();
            int res = usersManager.addUser("xminana",
                "otherpassword",
                "other@uoc.edu",
                "other",
                UserType.WORKER);
        }

        [TestMethod]
        [ExpectedException(typeof(GestorDespesesException))]
        public void createDuplicatedEmailUser()
        {
            UserBI usersManager = new UserBI();
            int res = usersManager.addUser("other",
                "otherpassword",
                "xminana@uoc.edu",
                "other",
                UserType.WORKER);
        }

        [TestMethod]
        public void deleteUser()
        {
            UserBI usersManager = new UserBI();
            bool res = usersManager.deleteUser(generatedUser);
            Assert.AreNotEqual(res, false);
        }

        [TestMethod]
        public void testHeadAssignment()
        {
            UserBI usersManager = new UserBI();
            user1Id = usersManager.addUser("user1", "password", "user1@uoc.edu", "user1", UserType.WORKER);
            Assert.AreNotEqual(user1Id, -1);
            user2Id = usersManager.addUser("user2", "password", "user2@uoc.edu", "user2", UserType.WORKER);
            Assert.AreNotEqual(user2Id, -1);

            bool res = usersManager.assignHead(user2Id, user1Id);
            Assert.AreEqual(res, true);

            User user1 = usersManager.getUser(user1Id);
            Assert.AreNotEqual(user1, null);
            User user2 = usersManager.getUser(user2Id);
            Assert.AreNotEqual(user2, null);

            Assert.AreEqual(user1.Workers.Contains(user2), true);
            Assert.AreNotEqual(user2.Head, user1Id);

            res = usersManager.deleteUser(user1Id);
            Assert.AreEqual(res, true);
            res = usersManager.deleteUser(user2Id);
            Assert.AreEqual(res, true);
        }
    }
}   
