﻿using GestorDespesesWeb.GestorDespesesWS;
using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Web;
using System.Web.Services;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace GestorDespesesWeb.Admin
{
    public partial class SystemProperties : System.Web.UI.Page
    {
        private static BasicHttpBinding binding = new BasicHttpBinding();
        private static EndpointAddress address = new EndpointAddress("http://192.168.1.75/GestorDespesesWS/GestorDespesesWS.svc");
        private static GestorDespesesWSClient client = new GestorDespesesWSClient(binding, address);

        protected void Page_Load(object sender, EventArgs e)
        {
            try
            {
                if (Session["userId"] == null || (UserType)Session["userType"] != UserType.ADMINISTRATOR)
                {
                    Response.Redirect("/Error/Error.aspx", false);
                }
                else
                {
                    List<ExpenseTypeData> sheets = client.listExpenseTypes((string)Session["token"]);
                    gwExpensesTypes.DataSource = sheets;
                    gwExpensesTypes.DataBind();
                    gwExpensesTypes.HeaderRow.TableSection = TableRowSection.TableHeader;
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                if (fedf.Detail.Message.Equals("Token de sessió invàlid."))
                {
                    Response.Redirect("/Account/Login.aspx", false);
                }
                else
                {
                    Response.Redirect("/Error/UnexpectedError.aspx", false);
                }
            }
            catch
            {
                Response.Redirect("/Error/UnexpectedError.aspx", false);
            }
        }

        protected void gwExpensesTypes_RowDataBound(object sender, GridViewRowEventArgs e)
        {
            if (e.Row.RowType == DataControlRowType.DataRow)
            {
                ExpenseTypeData expenseType = e.Row.DataItem as ExpenseTypeData;
                switch (expenseType.Type)
                {
                    case ExpenseTypeType.FIXED:
                        e.Row.Cells[2].Text = "Fixe";
                        break;
                    case ExpenseTypeType.VARIABLE:
                        e.Row.Cells[2].Text = "Valiable";
                        break;
                    case ExpenseTypeType.MULTIPLICITY:
                        e.Row.Cells[2].Text = "Multiplicatiu";
                        break;
                }
                if (expenseType.Value == null)
                    e.Row.Cells[3].Text = "--";
                if (expenseType.MaxValue == null)
                    e.Row.Cells[4].Text = "--";
                if (expenseType.Threshold == null)
                    e.Row.Cells[5].Text = "--";
            }
        }

        [WebMethod(EnableSession = true)]
        public static object getExpenseType(int expenseTypeId)
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.ADMINISTRATOR)
                {
                    ExpenseTypeData etype = client.getExpenseType(expenseTypeId, (string)HttpContext.Current.Session["token"]);
                    result = new { success = true, expenseType = etype };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object deleteExpenseType(int expenseTypeId)
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.ADMINISTRATOR)
                {
                    bool deleted = client.deleteExpenseType(expenseTypeId, (string)HttpContext.Current.Session["token"]);
                    result = new { success = deleted };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object addEditExpenseType(int expenseTypeId, string name, decimal value, decimal maxValue, decimal threshold, int type)
        {
            object result;
            try
            {
                //Get the user for check if it exists
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.ADMINISTRATOR)
                {
                    if (expenseTypeId > 0) //Edit
                    {
                        bool bresult = client.editExpenseType(expenseTypeId, name, (value != -1) ? value : default(decimal?), (maxValue != -1) ? maxValue : default(decimal?), (threshold != -1) ? threshold : default(decimal?), (ExpenseTypeType)type, (string)HttpContext.Current.Session["token"]);
                        if (bresult)
                        {
                            result = new { success = true, expenseTypeId = expenseTypeId };
                        }
                        else
                        {
                            result = new { success = false, error = "Error desconegut" };
                        }
                    }
                    else //Add 
                    {
                        int newExpenseTypeId = client.addExpenseType(name, (value != -1) ? value : default(decimal?), (maxValue != -1) ? maxValue : default(decimal?), (threshold != -1) ? threshold : default(decimal?), (ExpenseTypeType)type, (string)HttpContext.Current.Session["token"]);
                        result = new { success = true, expenseTypeId = newExpenseTypeId };
                    }
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object changeSMTPSettings(string host, int port, bool tls, string from, string username, string password)
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.ADMINISTRATOR)
                {
                    bool res = client.configureSMTP(from, username, password, host, port, tls, (string)HttpContext.Current.Session["token"]);
                    result = new { success = res };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object getSMTPSettings()
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.ADMINISTRATOR)
                {
                    SMTPData res = client.getSMTPSettings((string)HttpContext.Current.Session["token"]);
                    result = new { success = true, smtp = res };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object getCalendarInfo(int year, int month)
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.ADMINISTRATOR)
                {
                    List<DayData> dates = client.listMonthCalendar(year, month, (string)HttpContext.Current.Session["token"]);
                    result = new { success = true, dates = dates };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object setDayType(int year, int month, int day, int type)
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.ADMINISTRATOR)
                {
                    bool res = false;
                    DateTime date = new DateTime(year, month, day);
                    switch (type) { 
                        case 0: //Normal day
                            res= client.deleteCalendarValidationLimit(date, (string)HttpContext.Current.Session["token"]);
                            res= client.deleteCalendarHoliday(date, (string)HttpContext.Current.Session["token"]);
                            result = new { success = res };
                            break;
                        case 1: //festiu
                            res= client.setCalendarHoliday(date, (string)HttpContext.Current.Session["token"]);
                            result = new { success = res };
                            break;
                        case 2: //Last day
                            res= client.setCalendarValidationLimit(date, (string)HttpContext.Current.Session["token"]);
                            result = new { success = res };
                            break;
                        case 3: //Festiu i last day
                        default:
                            res= client.setCalendarHoliday(date, (string)HttpContext.Current.Session["token"]);
                            if (res)
                            {
                                res = client.setCalendarValidationLimit(date, (string)HttpContext.Current.Session["token"]);
                                result = new { success = res };
                            }
                            else 
                            {
                                result = new { success = false };
                            }
                            break;
                    }
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }
    }
}
