﻿using GestorDespesesWeb.GestorDespesesWS;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.Serialization.Json;
using System.ServiceModel;
using System.Web;
using System.Web.Script.Services;
using System.Web.Services;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace GestorDespesesWeb.Admin
{
    public partial class Users : System.Web.UI.Page
    {
        private static BasicHttpBinding binding = new BasicHttpBinding();
        private static EndpointAddress address = new EndpointAddress("http://192.168.1.75/GestorDespesesWS/GestorDespesesWS.svc");
        private static GestorDespesesWSClient client = new GestorDespesesWSClient(binding, address);

        protected void Page_Load(object sender, EventArgs e)
        {
            try
            {
                if (Session["userId"] == null || (UserType)Session["userType"] != UserType.ADMINISTRATOR)
                {
                    Response.Redirect("/Error/Error.aspx", false);
                }
                else
                {
                    List<UserData> users = client.listUsers((string)Session["token"]);
                    gwUsers.DataSource = users;
                    gwUsers.DataBind();
                    gwUsers.HeaderRow.TableSection = TableRowSection.TableHeader;
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                if (fedf.Detail.Message.Equals("Token de sessió invàlid."))
                {
                    Response.Redirect("/Account/Login.aspx", false);
                }
                else
                {
                    Response.Redirect("/Error/UnexpectedError.aspx", false);
                }
            }
            catch
            {
                Response.Redirect("/Error/UnexpectedError.aspx", false);
            }
        }

        protected void gwUsers_RowDataBound(object sender, GridViewRowEventArgs e)
        {
            if (e.Row.RowType == DataControlRowType.DataRow)
            {
                UserData user = e.Row.DataItem as UserData;
                switch (user.Type)
                {
                    case UserType.ADMINISTRATOR:
                        e.Row.Cells[2].Text = "Administrador";
                        break;
                    case UserType.WORKER:
                        if (user.WorkerHandlers.Count > 0)
                        {
                            e.Row.Cells[2].Text = "Responsable";
                        }
                        else
                        {
                            e.Row.Cells[2].Text = "Treballador";
                        }
                        break;
                }
            }
        }

        [WebMethod(EnableSession = true)]
        public static object getUserList()
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.ADMINISTRATOR)
                {
                    //Bind users to gridView
                    List<UserData> list = client.listUsers((string)HttpContext.Current.Session["token"]);
                    result = new { success = true, list = list };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession=true)]
        public static object getUsers(string searchParam)
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.ADMINISTRATOR)
                {
                    List<UserData> list = client.listUsersByQuery(searchParam, 25, (string)HttpContext.Current.Session["token"]);
                    result = new { success = true, list = list};
                }
                else
                {
                    result = new { success = false, error = "No autoritzat."};
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object getUser(int userId)
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.ADMINISTRATOR)
                {
                    UserData user = client.getUser(userId, (string)HttpContext.Current.Session["token"]);
                    result = new { success = true, user = user };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object deleteUser(int userId)
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.ADMINISTRATOR)
                {
                    bool deleted = client.deleteUser(userId, (string)HttpContext.Current.Session["token"]);
                    result = new { success = deleted };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object addEditUser(int userId, string username, string  password, string email, string fullName, int type, int headId)
        {
            object result;
            try
            {
                //Get the user for check if it exists
                UserData headUser = null;
                if (headId > 0)
                {
                    headUser = client.getUser(headId, (string)HttpContext.Current.Session["token"]);
                }
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.ADMINISTRATOR)
                {
                    if (userId > 0) //Edit
                    {
                        bool bresult = client.editUser(userId, password, email, fullName, (UserType)type, (string)HttpContext.Current.Session["token"]);
                        if (bresult)
                        {
                            result = new { success = true, userId = userId };
                            if (headUser != null)
                            {
                                client.assignHead(userId, headUser.UserHandler, (string)HttpContext.Current.Session["token"]);
                            }
                        }
                        else
                        {
                            result = new { success = false, error = "Error desconegut" };
                        }
                    }
                    else //Add 
                    { 
                        int newUserId = client.addUser(username, password, email, fullName, (UserType)type, (string)HttpContext.Current.Session["token"]);
                        if (newUserId > 0)
                        {
                            result = new { success = true, userId = newUserId };
                            if (headUser != null)
                            {
                                client.assignHead(newUserId, headUser.UserHandler, (string)HttpContext.Current.Session["token"]);
                            }
                        }
                        else
                        {
                            result = new { success = false, error = "Error desconegut" };
                        }

                    }
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }
    }
}