﻿using GestorDespesesWeb.GestorDespesesWS;
using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Web;
using System.Web.Script.Services;
using System.Web.Services;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace GestorDespesesWeb.Head
{
    public partial class Validation : System.Web.UI.Page
    {
        private static BasicHttpBinding binding = new BasicHttpBinding();
        private static EndpointAddress address = new EndpointAddress("http://192.168.1.75/GestorDespesesWS/GestorDespesesWS.svc");
        private static GestorDespesesWSClient client = new GestorDespesesWSClient(binding, address);

        protected void Page_Load(object sender, EventArgs e)
        {
            try
            {
                if (Session["userId"] == null || ((UserType)Session["userType"] != UserType.WORKER && (UserType)Session["userType"] != UserType.ADMINISTRATOR))
                {
                    Response.Redirect("/Error/Error.aspx", false);
                }
                else
                {
                    List<ExpensesSheetData> sheets = client.getExpensesSheetsToValidate((string)Session["token"]);
                    gwExpensesSheets.DataSource = sheets;
                    gwExpensesSheets.DataBind();
                    gwExpensesSheets.HeaderRow.TableSection = TableRowSection.TableHeader;
                    List<ExpenseData> expenses = new List<ExpenseData>();
                    gwExpenses.DataSource = expenses;
                    gwExpenses.DataBind();
                    gwExpenses.HeaderRow.TableSection = TableRowSection.TableHeader;
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                if (fedf.Detail.Message.Equals("Token de sessió invàlid."))
                {
                    Response.Redirect("/Account/Login.aspx", false);
                }
                else
                {
                    Response.Redirect("/Error/UnexpectedError.aspx", false);
                }
            }
            catch
            {
                Response.Redirect("/Error/UnexpectedError.aspx", false);
            }
        }

        protected void gwExpensesSheets_RowDataBound(object sender, GridViewRowEventArgs e)
        {
            if (e.Row.RowType == DataControlRowType.DataRow)
            {
                ExpensesSheetData expenseSheet = e.Row.DataItem as ExpensesSheetData;
                e.Row.Cells[3].Text = Utils.FormatDecimal(expenseSheet.Value);
            }
        }

        [WebMethod(EnableSession = true)]
        public static object getExpensesOfSheet(int expensesSheetId)
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && 
                    ((UserType)HttpContext.Current.Session["userType"] == UserType.WORKER || (UserType)HttpContext.Current.Session["userType"] == UserType.ADMINISTRATOR))
                {
                    List<ExpenseData> list = client.getExpensesOfExpensesSheet(expensesSheetId, (string)HttpContext.Current.Session["token"]);
                    result = new { success = true, list = list };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object validateExpensesSheet(int expensesSheetId)
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null &&
                    ((UserType)HttpContext.Current.Session["userType"] == UserType.WORKER || (UserType)HttpContext.Current.Session["userType"] == UserType.ADMINISTRATOR))
                {
                    bool deleted = client.validateExpensesSheet(expensesSheetId, (string)HttpContext.Current.Session["token"]);
                    result = new { success = deleted };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object revokeExpensesSheet(int expensesSheetId, string reason)
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null &&
                    ((UserType)HttpContext.Current.Session["userType"] == UserType.WORKER || (UserType)HttpContext.Current.Session["userType"] == UserType.ADMINISTRATOR))
                {
                    bool deleted = client.revokeExpensesSheet(expensesSheetId, reason, (string)HttpContext.Current.Session["token"]);
                    result = new { success = deleted };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }
    }
}
