﻿using GestorDespesesWeb.GestorDespesesWS;
using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Web;
using System.Web.Services;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace GestorDespesesWeb.Worker
{
    public partial class Expenses : System.Web.UI.Page
    {
        private static BasicHttpBinding binding = new BasicHttpBinding();
        private static EndpointAddress address = new EndpointAddress("http://192.168.1.75/GestorDespesesWS/GestorDespesesWS.svc");
        private static GestorDespesesWSClient client = new GestorDespesesWSClient(binding, address);

        protected void Page_Load(object sender, EventArgs e)
        {
            try
            {
                if (Session["userId"] == null || (UserType)Session["userType"] != UserType.WORKER)
                {
                    Response.Redirect("/Error/Error.aspx", false);
                }
                else
                {
                    ExpensesSheetState state = client.getCurrentExpensesState((string)Session["token"]); 
                    {
                        if (state.Equals(ExpensesSheetState.OPEN))
                        {
                            informationDiv.Visible = false;
                            revokedJS.Visible = false;
                            expensesDiv.Visible = true;
                            expensesJS.Visible = true;
                            expensesModals.Visible = true;
                            //Bind current expenses to gridView
                            List<ExpenseData> expenses = client.getCurrentExpenses((string)Session["token"]);
                            gwExpenses.DataSource = expenses;
                            gwExpenses.DataBind();
                            gwExpenses.HeaderRow.TableSection = TableRowSection.TableHeader;
                            decimal total = 0;
                            foreach(ExpenseData ed in expenses)
                            {
                                total += ed.TotalValue;
                            }
                            totalValueP.InnerText = "Total: " + Utils.FormatDecimal(total) + "€";

                        }
                        else
                        {
                            informationDiv.Visible = true;
                            expensesDiv.Visible = false;
                            expensesJS.Visible = false;
                            expensesModals.Visible = true;
                            if (state.Equals(ExpensesSheetState.REVOKED))
                            {
                                revokedJS.Visible = true;
                                string revokedReason = client.getCurrentExpensesRevokedReason((string)Session["token"]);
                                // Mostrar missatge de per que s'ha revocat.
                                InfoTitle.Text = "Despeses denegades";
                                Info1.Text = "Missatge del responsable: ";
                                Info2.Text = String.IsNullOrEmpty(revokedReason) ? "" : revokedReason;
                            }
                            else
                            {
                                reopenSheet.Visible = false;
                                revokedJS.Visible = false;
                                // Mostrar missatge de que no es pot fer res.
                                InfoTitle.Text = "Pendent de validació";
                                Info1.Text = "";
                                Info2.Text = "No pots introduir més despeses fins que el full de despeses hagi estat validat.";
                            }
                        }
                    }
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                if (fedf.Detail.Message.Equals("Token de sessió invàlid."))
                {
                    Response.Redirect("/Account/Login.aspx", false);
                }
                else
                {
                    Response.Redirect("/Error/UnexpectedError.aspx", false);
                }
            }
            catch
            {
                Response.Redirect("/Error/UnexpectedError.aspx");
            }
        }


        protected void gwExpenses_RowDataBound(object sender, GridViewRowEventArgs e)
        {
            if (e.Row.RowType == DataControlRowType.DataRow)
            {
                ExpenseData expense = e.Row.DataItem as ExpenseData;
                switch (expense.Type)
                {
                    case ExpenseTypeType.FIXED:
                        e.Row.Cells[3].Text = "Fixe";
                        break;
                    case ExpenseTypeType.MULTIPLICITY:
                        e.Row.Cells[3].Text = "Multiplicatiu";
                        break;
                    case ExpenseTypeType.VARIABLE:
                        e.Row.Cells[3].Text = "Variable";
                        break;
                }
                e.Row.Cells[4].Text = expense.Date.Day.ToString("D2") + "/" + expense.Date.Month.ToString("D2") + "/" + expense.Date.Year;
                e.Row.Cells[5].Text = (expense.TypeValue) == null ? "" : Utils.FormatDecimal(expense.TypeValue.Value);
                e.Row.Cells[6].Text = expense.IsOnHolidays ? "Si" : "No";
                e.Row.Cells[7].Text = expense.HasInvoice ? "Si" : "No";
                e.Row.Cells[8].Text = (expense.Value)==null ? "" : Utils.FormatDecimal(expense.Value.Value);
                e.Row.Cells[9].Text = Utils.FormatDecimal(expense.TotalValue);
            }
        }

        [WebMethod(EnableSession = true)]
        public static object getExpenses()
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.WORKER)
                {
                    //Bind current expenses to gridView
                    List<ExpenseData> list = client.getCurrentExpenses((string)HttpContext.Current.Session["token"]);
                    result = new { success = true, list = list };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object getExpense(int expenseHandler)
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.WORKER)
                {
                    //Bind current expenses to gridView
                    ExpenseData expense = client.getExpense(expenseHandler, (string)HttpContext.Current.Session["token"]);
                    string b64Invoice = null;
                    if(expense.HasInvoice == true)
                    {
                        byte[] invoice = client.getExpenseInvoice(expenseHandler, (string)HttpContext.Current.Session["token"]);
                        b64Invoice = Convert.ToBase64String(invoice);
                    }
                    result = new { success = true, expense = expense, b64Invoice = b64Invoice };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object getExpenseTypes(string searchParam)
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.WORKER)
                {
                    List<ExpenseTypeData> list = client.listExpenseTypesByQuery(searchParam, 25, (string)HttpContext.Current.Session["token"]);
                    result = new { success = true, list = list };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object deleteExpense(int expenseId)
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.WORKER)
                {
                    bool deleted = client.deleteExpense(expenseId, (string)HttpContext.Current.Session["token"]);
                    result = new { success = deleted };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object validateExpensesSheet()
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.WORKER)
                {
                    bool prevalid = client.preValidateCurrentExpenses((string)HttpContext.Current.Session["token"]);
                    result = new { success = prevalid };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object reopenExpensesSheet()
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.WORKER)
                {
                    bool prevalid = client.reOpenCurrentExpenses((string)HttpContext.Current.Session["token"]);
                    result = new { success = prevalid };
                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }

        [WebMethod(EnableSession = true)]
        public static object addExpense(int value)
        {
            object result;
            try
            {
                if (HttpContext.Current.Session["userId"] != null && (UserType)HttpContext.Current.Session["userType"] == UserType.WORKER)
                {
                    if(value > 10)
                        result = new { success = true };
                    else
                        result = new { success = false };

                }
                else
                {
                    result = new { success = false, error = "No autoritzat." };
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                result = new { success = false, error = fedf.Detail.Message };
            }
            catch (Exception)
            {
                result = new { success = false, error = "Error desconegut." };
            }
            return result;
        }
    }
}
