﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Tasks;
using System.ServiceModel;
using GestorDespesesWP7.GestorDespesesWS;
using System.Windows.Media.Imaging;
using System.IO;

namespace GestorDespesesWP7
{
    public partial class AddEditExpensePage : PhoneApplicationPage
    {
        private static BasicHttpBinding binding = new BasicHttpBinding();
        private static EndpointAddress address = new EndpointAddress(((App)Application.Current).ServiceUrl);
        private GestorDespesesWSClient client = null; 
        private static PhotoChooserTask photoChooserTask;
        private List<string> tipos = null;

        private byte[] bTiquet = null;
        private Dictionary<string,ExpenseTypeData> expenseTypes = null;

        //For parent control.
        public bool isExpenseAddedOrEdited = false;
        public bool isAddMode = false;
        public ExpensesListItem itemToEdit = null;
        public bool fromParent = false;

        public AddEditExpensePage()
        {
            InitializeComponent();
//            this.Loaded += new RoutedEventHandler(AddEditExpensePage_Loaded);
            lbUserFullName.Text = ((App)Application.Current).fullName;
            photoChooserTask = new PhotoChooserTask();
            photoChooserTask.ShowCamera = true;
            photoChooserTask.Completed +=new EventHandler<PhotoResult>(photoChooserTask_Completed);
            SetInputScope(expenseValue);

            client = new GestorDespesesWSClient(binding, address);
            client.listExpenseTypesCompleted +=new EventHandler<listExpenseTypesCompletedEventArgs>(client_listExpenseTypesCompleted);
            client.addExpenseCompleted += new EventHandler<addExpenseCompletedEventArgs>(client_addEditExpenseCompleted);
            client.editExpenseCompleted += new EventHandler<editExpenseCompletedEventArgs>(client_addEditExpenseCompleted);
            client.getExpenseInvoiceCompleted += new EventHandler<getExpenseInvoiceCompletedEventArgs>(client_getExpenseInvoiceCompleted);


            tipos = new List<string>();
            setExpenseTypesList();
        }

//        public void AddEditExpensePage_Loaded(object sender, RoutedEventArgs e)
        public void setMode()
        {
            if (fromParent)
            {
                if (isAddMode || itemToEdit == null)
                {
                    // Set default type selection changed state.
                    expenseValue.IsEnabled = true;
                    expenseValue.Text = "";
                    expenseValue.Width = 363;
                    lbValue.Text = "Value";
                    lbValue2.Text = "";

                    PageTitle.Text = "afegir despesa";
                    expenseDescription.Text = "";
                    expenseDate.Value = null;
                    //expenseType.Text = "";
                    expenseType.SelectedItem = null;
                }
                else
                {
                    PageTitle.Text = "editar despesa";
                    int indexOfType = tipos.IndexOf(itemToEdit.Concept.Text);
                    if (indexOfType >= 0)
                    {
                        expenseType.SelectedItem = tipos[indexOfType];
                    }
                    else
                    {
                        MessageBox.Show("El tipus de despesa ha cambiat i ja no està disponible. Seleccioni un tipus nou", "Informació", MessageBoxButton.OK);
                        // Set default type selection changed state.
                        expenseValue.IsEnabled = true;
                        expenseValue.Text = "";
                        expenseValue.Width = 363;
                        lbValue.Text = "Value";
                        lbValue2.Text = ""; 
                        expenseType.SelectedItem = null;
                    }
                    expenseDate.Value = DateTime.Parse(itemToEdit.Date.Text); // (Convert.ToInt32(parts[2]), Convert.ToInt32(parts[0]), Convert.ToInt32(parts[1]));
                    expenseDescription.Text = itemToEdit.Description.Text;
                    expenseValue.Text = itemToEdit.Value2.Text;
                    //Load image from expense
                    if (itemToEdit.HasImage.Text == "Si")
                    {
                        getExpenseTiquet(itemToEdit.ExpenseHandler);
                    }
                    this.Focus();
                }
            }
        }

        #region types list initialization

        private void setExpenseTypesList()
        {
            try
            {
                loading(true);
                client.listExpenseTypesAsync(((App)Application.Current).userToken);
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                loading(false);
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                loading(false);
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }        
        }

        private void client_listExpenseTypesCompleted(object sender, GestorDespesesWS.listExpenseTypesCompletedEventArgs e)
        {
            try
            {
                tipos.Clear();
                expenseTypes = new Dictionary<string, ExpenseTypeData>();
                foreach (ExpenseTypeData expenseTypeItem in e.Result)
                {
                    tipos.Add(expenseTypeItem.Concept);
                    expenseTypes.Add(expenseTypeItem.Concept, expenseTypeItem);
                }
                expenseType.ItemsSource = tipos;
                expenseType.PopulateComplete();
                setMode();
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false);
            }
        }

        #endregion

        private void expensePhotoButton_Click(object sender, EventArgs e)
        {
            photoChooserTask.Show();
        }

        private void expenseOKButton_Click(object sender, EventArgs e)
        {
            //Create a new element and set it to global
            //((App)Application.Current).addedExpenseListItemSelected = null;
            decimal tmp;
            if (String.IsNullOrEmpty(expenseDescription.Text))
            {
                MessageBox.Show("Has de facilitar una descripció de la despesa", "Error", MessageBoxButton.OK);
            }
            else if(expenseType.SelectedItem == null)
            {
                MessageBox.Show("Has de seleccionar un tipus de despesa", "Error", MessageBoxButton.OK);
            }
            else if (expenseDate.Value == null)
            {
                MessageBox.Show("Has de facilitar la data de la despesa", "Error", MessageBoxButton.OK);
            }
            else if (String.IsNullOrEmpty(expenseValue.Text))
            {
                MessageBox.Show("Has d'introduïr un valor/quantitat de la despesa", "Error", MessageBoxButton.OK);
            }
            else if (!decimal.TryParse(expenseValue.Text, out tmp))
            {
                MessageBox.Show("El valor introduït no és correcte", "Error", MessageBoxButton.OK);
            }
            else
            {
                try
                {
                    loading(true);
                    if (isAddMode)
                    {
                        client.addExpenseAsync(tmp, (DateTime)expenseDate.Value, expenseDescription.Text, bTiquet, (string)expenseType.SelectedItem, ((App)Application.Current).userToken);
                    }
                    else
                    {
                        client.editExpenseAsync(itemToEdit.ExpenseHandler, tmp, (DateTime)expenseDate.Value, expenseDescription.Text, bTiquet, (string)expenseType.SelectedItem, ((App)Application.Current).userToken);
                    }
                }
                catch (FaultException<GestorDespesesFault> fedf)
                {
                    loading(false);
                    MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
                }
                catch
                {
                    loading(false);
                    MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
                }   

            }
        }

        private void client_addEditExpenseCompleted(object sender, EventArgs e) 
        {
            try
            {
                if (isAddMode)
                {
                    if (((addExpenseCompletedEventArgs)e).Result > 0)
                    {
                        isExpenseAddedOrEdited = true;
                        NavigationService.GoBack();
                    }
                }
                else
                {
                    if (((editExpenseCompletedEventArgs)e).Result == true)
                    {
                        isExpenseAddedOrEdited = true;
                        NavigationService.GoBack();
                    }
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false);
            }
        }

        private void expenseCancelButton_Click(object sender, EventArgs e)
        {
            isExpenseAddedOrEdited = false;
            NavigationService.GoBack();
        }

        private void getExpenseTiquet(int expenseId)
        {
            try
            {
                loading(true);
                client.getExpenseInvoiceAsync(expenseId, ((App)Application.Current).userToken);
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                loading(false);
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                loading(false);
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }        
        }


        private void client_getExpenseInvoiceCompleted(object sender, GestorDespesesWS.getExpenseInvoiceCompletedEventArgs e)
        {
            try
            {
                if (e.Result != null)
                {
                    // load 'imageBytes' byte array to data base ...
                    BitmapImage bmp = new BitmapImage();
                    MemoryStream ms = new MemoryStream(e.Result);
                    bmp.SetSource(ms);
                    expenseTiquet.Source = bmp;
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false);
            }
        }


        private void photoChooserTask_Completed(object sender, PhotoResult e)
        {
            if (e.TaskResult == TaskResult.OK)
            {
                BitmapImage bmp = new BitmapImage();
                bmp.CreateOptions = BitmapCreateOptions.None;
                bmp.SetSource(e.ChosenPhoto);
                expenseTiquet.Source = bmp;
                ScaleTransform st = new ScaleTransform()
                {
                    ScaleX = 0.5,
                    ScaleY = 0.5
                };
                WriteableBitmap wTiquet = new WriteableBitmap(expenseTiquet, st);
                bTiquet = WritableBitmapToByteArray(wTiquet);
            }
        }

        private byte[] WritableBitmapToByteArray(WriteableBitmap wb) {
            byte[] data = null;
            if (wb != null)
            {
                using (MemoryStream stream = new MemoryStream())
                {
                    wb.SaveJpeg(stream, wb.PixelWidth, wb.PixelHeight, 0, 100);
                    stream.Seek(0, SeekOrigin.Begin);
                    data = stream.GetBuffer();
                }
            }
            return data;
        }

        private void PhoneApplicationPage_BackKeyPress(object sender, System.ComponentModel.CancelEventArgs e)
        {
            isExpenseAddedOrEdited = false;
            NavigationService.GoBack();
        }

        private void SetInputScope(TextBox textBoxControl)
        {
            InputScopeNameValue digitsInputNameScope = InputScopeNameValue.Number;
            textBoxControl.InputScope = new InputScope()
            {
                Names = { new InputScopeName() { NameValue = digitsInputNameScope } }
            };
        }

        protected override void OnNavigatedFrom(System.Windows.Navigation.NavigationEventArgs e)
        {
            // Per si un cas tornem a aquesta plana hem de resetejar el valor. Si tornem del parent ja es setejarà de nou
            fromParent = false;
            // Setejem la informació necessària per el parent.
            if (e.Content is ExpensesPivotPage)
            {
                (e.Content as ExpensesPivotPage).isExpenseAddedOrEdited = isExpenseAddedOrEdited;
            }

            base.OnNavigatedFrom(e);
        }

        private void loading(bool state)
        {
            if (!state)
            {
                rLoading.Hide();
            }
            else
            {
                rLoading.Show();
            }
        }

        private void expenseType_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (expenseType.SelectedItem != null)
            {
                ExpenseTypeData typeData = expenseTypes[(string)expenseType.SelectedItem];
                if (typeData != null)
                {
                    switch (typeData.Type)
                    {
                        case ExpenseTypeType.FIXED:
                            expenseValue.IsEnabled = false;
                            expenseValue.Text = Utils.FormatDecimal((decimal)typeData.Value);
                            expenseValue.Width = 363;
                            lbValue.Text = "Value";
                            lbValue2.Text = "";
                            break;
                        case ExpenseTypeType.MULTIPLICITY:
                            expenseValue.IsEnabled = true;
                            expenseValue.Text = "";
                            expenseValue.Width = 200;
                            lbValue.Text = "Quantitat";
                            lbValue2.Text = "x " + Utils.FormatDecimal((decimal)typeData.Value);
                            break;
                        case ExpenseTypeType.VARIABLE:
                            expenseValue.IsEnabled = true;
                            expenseValue.Text = "";
                            expenseValue.Width = 363;
                            lbValue.Text = "Value";
                            lbValue2.Text = "";
                            break;
                    }
                }
            }
            else 
            {
                expenseValue.IsEnabled = true;
                expenseValue.Text = "";
                expenseValue.Width = 363;
                lbValue.Text = "Value";
                lbValue2.Text = "";
            }
        }
    }
}