﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.Threading;
using System.ServiceModel;
using GestorDespesesWP7.GestorDespesesWS;
using Microsoft.Phone.Shell;
using System.Collections.ObjectModel;
using Coding4Fun.Toolkit.Controls;

namespace GestorDespesesWP7
{
    public partial class ExpensesPivotPage : PhoneApplicationPage
    {
        private static BasicHttpBinding binding = new BasicHttpBinding();
        private static EndpointAddress address = new EndpointAddress(((App)Application.Current).ServiceUrl);
        private GestorDespesesWSClient client = null; 
        private bool expensesLoaded = false;
        private bool historyLoaded = false;
        private bool validationLoaded = false;
        private ApplicationBar expensesApplicationBar = null;
        private ApplicationBar expensesValidatingApplicationBar = null;
        private ApplicationBar historyApplicationBar = null;
        private ApplicationBar validationApplicationBar = null;
        private ObservableCollection<ExpensesListItem> currentExpensesListCollection = null;
        private ObservableCollection<HistoryExpensesSheetListItem> historyExpensesSheetListCollection = null;
        private ObservableCollection<ValidationExpensesSheetListItem> validationExpensesSheetListCollection = null;
        private ExpensesSheetState currentExpensesSheetState = ExpensesSheetState.OPEN;
        
        // For addEditExpensePage control
        public bool isExpenseAddedOrEdited = false;
        public bool isAddMode = false;
        public ExpensesListItem itemToEdit = null;

        public ExpensesPivotPage()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(ExpensesPivotPage_Loaded);
            loading(false, 0);
            loading(false, 1);
            loading(false, 2);
            lbUserFullName.Text = ((App)Application.Current).fullName;
            currentExpensesListCollection = new ObservableCollection<ExpensesListItem>();
            currentExpensesList.ItemsSource = currentExpensesListCollection;

            historyExpensesSheetListCollection = new ObservableCollection<HistoryExpensesSheetListItem>();
            historyExpensesSheetList.ItemsSource = historyExpensesSheetListCollection;

            validationExpensesSheetListCollection = new ObservableCollection<ValidationExpensesSheetListItem>();
            validationExpensesSheetList.ItemsSource = validationExpensesSheetListCollection;

            //Carreguem la plana de despeses primer.
            ExpensesPivot.SelectedItem = expensesPivotItem;
            
            client = new GestorDespesesWSClient(binding, address);

            //current expenses events
            client.getCurrentExpensesStateCompleted += new EventHandler<getCurrentExpensesStateCompletedEventArgs>(client_getCurrentExpensesStateCompleted);
            client.getCurrentExpensesRevokedReasonCompleted += new EventHandler<getCurrentExpensesRevokedReasonCompletedEventArgs>(client_getCurrentExpensesRevokedReasonCompleted);
            client.reOpenCurrentExpensesCompleted += new EventHandler<reOpenCurrentExpensesCompletedEventArgs>(client_reOpenCurrentExpensesCompleted);
            client.isCurrentUserHeadUserCompleted += new EventHandler<isCurrentUserHeadUserCompletedEventArgs>(client_isCurrentUserHeadUserCompleted);
            client.getCurrentExpensesCompleted += new EventHandler<getCurrentExpensesCompletedEventArgs>(client_getCurrentExpensesCompleted);
            client.deleteExpenseCompleted += new EventHandler<deleteExpenseCompletedEventArgs>(client_deleteExpenseCompleted);
            client.preValidateCurrentExpensesCompleted += new EventHandler<preValidateCurrentExpensesCompletedEventArgs>(client_preValidateCurrentExpensesCompleted);

            //history events
            client.getExpenseSheetsHistoryCompleted += new EventHandler<getExpenseSheetsHistoryCompletedEventArgs>(client_getExpenseSheetsHistoryCompleted);

            //validation events
            client.getExpensesSheetsToValidateCompleted += new EventHandler<getExpensesSheetsToValidateCompletedEventArgs>(client_getExpensesSheetsToValidateCompleted);
            client.validateExpensesSheetCompleted += new EventHandler<validateExpensesSheetCompletedEventArgs>(client_validateExpensesSheetCompleted);
            client.revokeExpensesSheetCompleted += new EventHandler<revokeExpensesSheetCompletedEventArgs>(client_revokeExpensesSheetCompleted);

            expensesState();
            checkIfUserIsHeadUser();
            createExpensesApplicationBar();
            createExpensesValidationApplicationBar();
            createHistoryApplicationBar();
            createValidationApplicationBar();
        }

        private void ExpensesPivotPage_Loaded(object sender, RoutedEventArgs e)
        {
            if (((PivotItem)ExpensesPivot.SelectedItem).Name == "expensesPivotItem" && expensesOpen.Visibility == Visibility.Visible)
            {
                if (isExpenseAddedOrEdited)
                {
                    loadCurrentExpenses(true);
                }
            }
        }

        #region ApplicationBars

        private void createExpensesApplicationBar() 
        {
            if (expensesApplicationBar == null)
            {
                expensesApplicationBar = new ApplicationBar();
                expensesApplicationBar.Mode = ApplicationBarMode.Default;
                expensesApplicationBar.Opacity = 1.0;
                expensesApplicationBar.IsVisible = true;
                expensesApplicationBar.IsMenuEnabled = true;

                ApplicationBarIconButton addExpenseButton = new ApplicationBarIconButton();
                addExpenseButton.IconUri = new Uri("/Images/appbar.new.rest.png", UriKind.Relative);
                addExpenseButton.Text = "Afegir";
                expensesApplicationBar.Buttons.Add(addExpenseButton);
                addExpenseButton.Click += new EventHandler(addExpenseButton_Click);

                ApplicationBarIconButton editExpenseButton = new ApplicationBarIconButton();
                editExpenseButton.IconUri = new Uri("/Images/appbar.edit.rest.png", UriKind.Relative);
                editExpenseButton.Text = "Editar";
                expensesApplicationBar.Buttons.Add(editExpenseButton);
                editExpenseButton.Click += new EventHandler(editExpenseButton_Click);

                ApplicationBarIconButton deleteExpenseButton = new ApplicationBarIconButton();
                deleteExpenseButton.IconUri = new Uri("/Images/appbar.delete.rest.png", UriKind.Relative);
                deleteExpenseButton.Text = "Esborrar";
                expensesApplicationBar.Buttons.Add(deleteExpenseButton);
                deleteExpenseButton.Click += new EventHandler(deleteExpenseButton_Click);

                ApplicationBarIconButton refreshExpenseButton = new ApplicationBarIconButton();
                refreshExpenseButton.IconUri = new Uri("/Images/appbar.refresh.rest.png", UriKind.Relative);
                refreshExpenseButton.Text = "Recàrrega";
                expensesApplicationBar.Buttons.Add(refreshExpenseButton);
                refreshExpenseButton.Click += new EventHandler(refreshExpenseButton_Click);

                ApplicationBarMenuItem validateExpensesMenuItem = new ApplicationBarMenuItem();
                validateExpensesMenuItem.Text = "Validar full...";
                expensesApplicationBar.MenuItems.Add(validateExpensesMenuItem);
                validateExpensesMenuItem.Click += new EventHandler(validateExpensesMenuItem_Click);
            }
        }

        private void createExpensesValidationApplicationBar()
        {
            if (expensesValidatingApplicationBar == null)
            {
                expensesValidatingApplicationBar = new ApplicationBar();
                expensesValidatingApplicationBar.Mode = ApplicationBarMode.Default;
                expensesValidatingApplicationBar.Opacity = 1.0;
                expensesValidatingApplicationBar.IsVisible = true;
                expensesValidatingApplicationBar.IsMenuEnabled = false;

                ApplicationBarIconButton refreshExpenseButton2 = new ApplicationBarIconButton();
                refreshExpenseButton2.IconUri = new Uri("/Images/appbar.refresh.rest.png", UriKind.Relative);
                refreshExpenseButton2.Text = "Recàrrega";
                expensesValidatingApplicationBar.Buttons.Add(refreshExpenseButton2);
                refreshExpenseButton2.Click += new EventHandler(refreshExpenseButton_Click);
            }
        }

        private void createHistoryApplicationBar()
        {
            if (historyApplicationBar == null)
            {
                historyApplicationBar = new ApplicationBar();
                historyApplicationBar.Mode = ApplicationBarMode.Default;
                historyApplicationBar.Opacity = 1.0;
                historyApplicationBar.IsVisible = true;
                historyApplicationBar.IsMenuEnabled = false;

                ApplicationBarIconButton viewHistoryButton = new ApplicationBarIconButton();
                viewHistoryButton.IconUri = new Uri("/Images/appbar.feature.search.rest.png", UriKind.Relative);
                viewHistoryButton.Text = "Detalls";
                historyApplicationBar.Buttons.Add(viewHistoryButton);
                viewHistoryButton.Click += new EventHandler(viewHistoryButton_Click);

                ApplicationBarIconButton refreshHistoryButton = new ApplicationBarIconButton();
                refreshHistoryButton.IconUri = new Uri("/Images/appbar.refresh.rest.png", UriKind.Relative);
                refreshHistoryButton.Text = "Recàrrega";
                historyApplicationBar.Buttons.Add(refreshHistoryButton);
                refreshHistoryButton.Click += new EventHandler(refreshHistoryButton_Click);
            }
        }

        private void createValidationApplicationBar()
        {
            if (validationApplicationBar == null)
            {
                validationApplicationBar = new ApplicationBar();
                validationApplicationBar.Mode = ApplicationBarMode.Default;
                validationApplicationBar.Opacity = 1.0;
                validationApplicationBar.IsVisible = true;
                validationApplicationBar.IsMenuEnabled = false;

                ApplicationBarIconButton viewValidationButton = new ApplicationBarIconButton();
                viewValidationButton.IconUri = new Uri("/Images/appbar.feature.search.rest.png", UriKind.Relative);
                viewValidationButton.Text = "Detalls";
                validationApplicationBar.Buttons.Add(viewValidationButton);
                viewValidationButton.Click += new EventHandler(viewValidationButton_Click);

                ApplicationBarIconButton refreshValidationButton = new ApplicationBarIconButton();
                refreshValidationButton.IconUri = new Uri("/Images/appbar.refresh.rest.png", UriKind.Relative);
                refreshValidationButton.Text = "Recàrrega";
                validationApplicationBar.Buttons.Add(refreshValidationButton);
                refreshValidationButton.Click += new EventHandler(refreshValidationButton_Click);

                ApplicationBarIconButton validateExpenseSheetButton = new ApplicationBarIconButton();
                validateExpenseSheetButton.IconUri = new Uri("/Images/appbar.check.rest.png", UriKind.Relative);
                validateExpenseSheetButton.Text = "Validar";
                validationApplicationBar.Buttons.Add(validateExpenseSheetButton);
                validateExpenseSheetButton.Click += new EventHandler(validateExpenseSheetButton_Click);

                ApplicationBarIconButton revokeExpenseSheetButton = new ApplicationBarIconButton();
                revokeExpenseSheetButton.IconUri = new Uri("/Images/appbar.cancel.rest.png", UriKind.Relative);
                revokeExpenseSheetButton.Text = "Rebutjar";
                validationApplicationBar.Buttons.Add(revokeExpenseSheetButton);
                revokeExpenseSheetButton.Click += new EventHandler(revokeExpenseSheetButton_Click);
            }
        }

        #endregion

        private void lbExitButton_Click(object sender, RoutedEventArgs e)
        {
            if (MessageBoxResult.OK == MessageBox.Show("Està segur que vol sortir?", "Sortir", MessageBoxButton.OKCancel))
            {
                ((App)Application.Current).reset();
                NavigationService.Navigate(new Uri("/Login.xaml", UriKind.Relative));
            }
        }

        #region despeses

        private void expensesState()
        {
            try
            {
                loading(true, 0);
                client.getCurrentExpensesStateAsync(((App)Application.Current).userToken);
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                loading(false,0);
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                loading(false,0);
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
        }

        private void client_getCurrentExpensesStateCompleted(object sender, GestorDespesesWS.getCurrentExpensesStateCompletedEventArgs e)
        {
            try
            {
                if (currentExpensesSheetState != e.Result)
                {
                    historyLoaded = false; //Si detectem un cambi d'estat el history ha cambiat segur.
                    currentExpensesSheetState = e.Result;
                }
                switch (e.Result) {
                    case ExpensesSheetState.OPEN:
                        tbTotalValue.Visibility = Visibility.Visible;
                        expensesOpen.Visibility = Visibility.Visible;
                        expensesValidating.Visibility = Visibility.Collapsed;
                        expensesRevoked.Visibility = Visibility.Collapsed;
                        ApplicationBar = expensesApplicationBar;
                        loadCurrentExpenses(false);
                        break;
                    case ExpensesSheetState.VALIDATING:
                        tbTotalValue.Visibility = Visibility.Collapsed;
                        expensesOpen.Visibility = Visibility.Collapsed;
                        expensesValidating.Visibility = Visibility.Visible;
                        expensesRevoked.Visibility = Visibility.Collapsed;
                        ApplicationBar = expensesValidatingApplicationBar;
                        break;
                    case ExpensesSheetState.REVOKED:
                        tbTotalValue.Visibility = Visibility.Collapsed;
                        expensesOpen.Visibility = Visibility.Collapsed;
                        expensesValidating.Visibility = Visibility.Collapsed;
                        expensesRevoked.Visibility = Visibility.Visible;
                        ApplicationBar = null;
                        getRevokedReason();
                        break;
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false, 0);
            }
        }

        #region OPEN

        private void loadCurrentExpenses(bool force)
        {
            // Només carreguem si no està carregat prèviament o si forçem pitjan el botó de refresh
            if (force || !expensesLoaded)
            {
                try
                {
                    loading(true, 0);
                    client.getCurrentExpensesAsync(((App)Application.Current).userToken);
                }
                catch (FaultException<GestorDespesesFault> fedf)
                {
                    loading(false, 0);
                    MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
                }
                catch
                {
                    loading(false, 0);
                    MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
                }
            }
        }

        private void client_getCurrentExpensesCompleted(object sender, GestorDespesesWS.getCurrentExpensesCompletedEventArgs e)
        {
            try
            {
                decimal total = 0;
                currentExpensesListCollection.Clear();
                foreach (ExpenseData expense in e.Result)
                {
                    ExpensesListItem item = new ExpensesListItem();
                    item.ExpenseHandler = expense.ExpenseHandler;
                    item.Description.Text = expense.Description;
                    item.Value.Text = (expense.TypeValue) == null ? "-" : Utils.FormatDecimal(expense.TypeValue.Value);
                    item.Value2.Text = (expense.Value) == null ? "-" : Utils.FormatDecimal(expense.Value.Value);
                    item.Concept.Text = expense.TypeConcept;
                    item.Date.Text = expense.Date.ToShortDateString();
                    item.IsHolidays.Text = expense.IsOnHolidays ? "Si" : "No";
                    item.HasImage.Text = expense.HasInvoice ? "Si" : "No";
                    item.Total.Text = Utils.FormatDecimal(expense.TotalValue);
                    switch (expense.Type)
                    {
                        case ExpenseTypeType.FIXED:
                            item.Type.Text = "Fixe";
                            break;
                        case ExpenseTypeType.MULTIPLICITY:
                            item.Type.Text = "Multiplicatiu";
                            break;
                        case ExpenseTypeType.VARIABLE:
                            item.Type.Text = "Variable";
                            break;
                    }

                    currentExpensesListCollection.Add(item);
                    total += expense.TotalValue;
                }
                tbTotalValue.Text = "Total: " + Utils.FormatDecimal(total) + "€";
                expensesLoaded = true;
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false, 0);
            }
        }

        private void addExpenseButton_Click(object sender, EventArgs e)
        {
            isAddMode = true;
            itemToEdit = null;
            NavigationService.Navigate(new Uri("/AddEditExpensePage.xaml", UriKind.Relative));
        }

        private void editExpenseButton_Click(object sender, EventArgs e)
        {
            if (((App)Application.Current).currentExpenseListItemSelected == null)
            {
                MessageBox.Show("Has de seleccionar una despesa per poder editar-la", "Informació", MessageBoxButton.OK);
            }
            else
            {
                isAddMode = false;
                itemToEdit = ((App)Application.Current).currentExpenseListItemSelected;
                NavigationService.Navigate(new Uri("/AddEditExpensePage.xaml", UriKind.Relative));
            }
        }

        private void deleteExpenseButton_Click(object sender, EventArgs e)
        {
            if (((App)Application.Current).currentExpenseListItemSelected == null)
            {
                MessageBox.Show("Has de seleccionar una despesa per poder esborrar-la", "Informació", MessageBoxButton.OK);
            }
            else
            {
                if (MessageBoxResult.OK == MessageBox.Show("Està segur que vol esborrar la despesa seleccionada?", "Alerta", MessageBoxButton.OKCancel))
                {
                    try
                    {
                        loading(true, 0);
                        client.deleteExpenseAsync(((App)Application.Current).currentExpenseListItemSelected.ExpenseHandler, ((App)Application.Current).userToken);
                    }
                    catch (FaultException<GestorDespesesFault> fedf)
                    {
                        loading(false, 0);
                        MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
                    }
                    catch
                    {
                        loading(false, 0);
                        MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
                    }
                }
            }
        }

        private void client_deleteExpenseCompleted(object sender, GestorDespesesWS.deleteExpenseCompletedEventArgs e)
        {
            try
            {
                if (e.Result)
                {
                    currentExpensesListCollection.RemoveAt(currentExpensesList.SelectedIndex);
                    ((App)Application.Current).currentExpenseListItemSelected = null;
                    decimal total = 0;
                    foreach (ExpensesListItem item in currentExpensesList.Items)
                    {
                        decimal tmp;
                        decimal.TryParse(item.Total.Text, out tmp);
                        total += tmp;
                    }
                    tbTotalValue.Text = "Total: " + Utils.FormatDecimal(total) + "€";
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false, 0);
            }
        }

        private void refreshExpenseButton_Click(object sender, EventArgs e)
        {
            expensesLoaded = false; //Forçem la recàrrega
            expensesState();
        }

        private void validateExpensesMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBoxResult.OK == MessageBox.Show("Està segur que vol enviar a validar les despeses actuals?", "Alerta", MessageBoxButton.OKCancel))
            {
                try
                {
                    loading(true, 0);
                    client.preValidateCurrentExpensesAsync(((App)Application.Current).userToken);
                }
                catch (FaultException<GestorDespesesFault> fedf)
                {
                    loading(false, 0);
                    MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
                }
                catch
                {
                    loading(false, 0);
                    MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
                }
            }
        }

        private void client_preValidateCurrentExpensesCompleted(object sender, GestorDespesesWS.preValidateCurrentExpensesCompletedEventArgs e)
        {
            try
            {
                if (e.Result)
                {
                    expensesLoaded = false; //Forçem la recàrrega 
                    expensesState();
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false, 0);
            }
        }

        #endregion

        #region VALIDATING
        
        private void checkIfUserIsHeadUser()
        {
            try
            {
                loading(true, 0);
                client.isCurrentUserHeadUserAsync(((App)Application.Current).userToken);
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                loading(false, 0);
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                loading(false, 0);
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
        }

        private void client_isCurrentUserHeadUserCompleted(object sender, GestorDespesesWS.isCurrentUserHeadUserCompletedEventArgs e)
        {
            try
            {
                if(!e.Result)
                {
                    // No es usuari responsable, esborrem el pivotItem per usuaris responsables.
                    foreach (PivotItem item in ExpensesPivot.Items)
                    {
                        if (item.Name == "validationPivotItem") 
                        {
                            ExpensesPivot.Items.Remove(item);
                            break;
                        } 
                    }
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false, 0);
            }
        }
        
        #endregion

        #region REVOKED
        private void getRevokedReason()
        {
            try
            {
                loading(true, 0);
                client.getCurrentExpensesRevokedReasonAsync(((App)Application.Current).userToken);
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                loading(false, 0);
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                loading(false, 0);
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
        }

        private void client_getCurrentExpensesRevokedReasonCompleted(object sender, GestorDespesesWS.getCurrentExpensesRevokedReasonCompletedEventArgs e) 
        {
            try
            {
                lbRevokedMessage.Text = e.Result;
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false, 0);
            }
        }
        
        private void btRevokedReopen_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                loading(true, 0);
                client.reOpenCurrentExpensesAsync(((App)Application.Current).userToken);
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                loading(false, 0);
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                loading(false, 0);
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
        }

        private void client_reOpenCurrentExpensesCompleted(object sender, GestorDespesesWS.reOpenCurrentExpensesCompletedEventArgs e)
        {
            try
            {
                if(e.Result)
                {
                    //expensesLoaded = false; //Forçem la recàrrega.
                    expensesState();
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false, 0);
            }
        }

        #endregion
        
        #endregion

        #region historic

        private void loadHistory(bool force)
        {
            // Només carreguem si no està carregat prèviament o si forçem pitjan el botó de refresh
            if (force || !historyLoaded)
            {
                try
                {
                    loading(true, 1);
                    client.getExpenseSheetsHistoryAsync(((App)Application.Current).userToken);
                }
                catch (FaultException<GestorDespesesFault> fedf)
                {
                    loading(false, 1);
                    MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
                }
                catch
                {
                    loading(false, 1);
                    MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
                }
            }
        }

        private void client_getExpenseSheetsHistoryCompleted(object sender, GestorDespesesWS.getExpenseSheetsHistoryCompletedEventArgs e)
        {
            try
            {
                historyExpensesSheetListCollection.Clear();
                foreach (ExpensesSheetData expenseSheet in e.Result)
                {
                    HistoryExpensesSheetListItem item = new HistoryExpensesSheetListItem();
                    item.ExpensesSheetHandler = expenseSheet.ExpensesSheetHandler;
                    switch (expenseSheet.State)
                    {
                        case ExpensesSheetState.OPEN:
                            item.State.Text = "Obert";
                            break;
                        case ExpensesSheetState.REVOKED:
                            item.State.Text = "Rebutjat";
                            break;
                        case ExpensesSheetState.VALIDATING:
                            item.State.Text = "Validant";
                            break;
                        case ExpensesSheetState.VALIDATED:
                            item.State.Text = "Validat";
                            break;
                        case ExpensesSheetState.PAYED:
                            item.State.Text = "Pagat";
                            break;
                    }
                    item.DateTime.Text = expenseSheet.Date.ToShortDateString() + " " + expenseSheet.Date.ToLongTimeString();
                    item.Total.Text = Utils.FormatDecimal(expenseSheet.Value);
                    historyExpensesSheetListCollection.Add(item);
                }
                historyLoaded = true;
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false, 1);
            }
        }

        private void refreshHistoryButton_Click(object sender, EventArgs e)
        {
            loadHistory(true);
        }

        private void viewHistoryButton_Click(object sender, EventArgs e)
        {
            if (((App)Application.Current).historyExpensesSheetListItemSelected == null)
            {
                MessageBox.Show("Has de seleccionar un full per poder veure els detalls", "Informació", MessageBoxButton.OK);
            }
            else
            {
                NavigationService.Navigate(new Uri("/ExpensesSheetDetailsPage.xaml", UriKind.Relative));
            }
        }

        #endregion

        #region validacio

        private void loadValidation(bool force)
        {
            // Només carreguem si no està carregat prèviament o si forçem pitjan el botó de refresh
            if (force || !validationLoaded)
            {
                try
                {
                    loading(true, 2);
                    client.getExpensesSheetsToValidateAsync(((App)Application.Current).userToken);
                }
                catch (FaultException<GestorDespesesFault> fedf)
                {
                    loading(false, 2);
                    MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
                }
                catch
                {
                    loading(false, 2);
                    MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
                }
            }
        }

        private void client_getExpensesSheetsToValidateCompleted(object sender, GestorDespesesWS.getExpensesSheetsToValidateCompletedEventArgs e)
        {
            try
            {
                validationExpensesSheetListCollection.Clear();
                foreach (ExpensesSheetData expenseSheet in e.Result)
                {
                    ValidationExpensesSheetListItem item = new ValidationExpensesSheetListItem();
                    item.ExpensesSheetHandler = expenseSheet.ExpensesSheetHandler;
                    item.FullUserName.Text = expenseSheet.OwnerUsername;
                    item.DateTime.Text = expenseSheet.Date.ToShortDateString() + " " + expenseSheet.Date.ToLongTimeString();
                    item.Total.Text = Utils.FormatDecimal(expenseSheet.Value);
                    validationExpensesSheetListCollection.Add(item);
                }
                validationLoaded = true;
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false, 2);
            }
        }

        private void refreshValidationButton_Click(object sender, EventArgs e)
        {
            loadValidation(true);
        }

        private void viewValidationButton_Click(object sender, EventArgs e)
        {
            if (((App)Application.Current).validationExpensesSheetListItemSelected == null)
            {
                MessageBox.Show("Has de seleccionar un full per poder veure els detalls", "Informació", MessageBoxButton.OK);
            }
            else
            {
                NavigationService.Navigate(new Uri("/ExpensesSheetDetailsPage.xaml", UriKind.Relative));
            }
        }

        private void validateExpenseSheetButton_Click(object sender, EventArgs e)
        {
            if (((App)Application.Current).validationExpensesSheetListItemSelected == null)
            {
                MessageBox.Show("Has de seleccionar un full per poder validar-lo", "Informació", MessageBoxButton.OK);
            }
            else
            {
                if (MessageBoxResult.OK == MessageBox.Show("Està segur que vol validar el full seleccionat?", "Alerta", MessageBoxButton.OKCancel))
                {
                    try
                    {
                        loading(true, 2);
                        client.validateExpensesSheetAsync(((App)Application.Current).validationExpensesSheetListItemSelected.ExpensesSheetHandler ,((App)Application.Current).userToken);
                    }
                    catch (FaultException<GestorDespesesFault> fedf)
                    {
                        loading(false, 2);
                        MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
                    }
                    catch
                    {
                        loading(false, 2);
                        MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
                    }
                }
            }
        }

        private void client_validateExpensesSheetCompleted(object sender, GestorDespesesWS.validateExpensesSheetCompletedEventArgs e)
        {
            try
            {
                if (e.Result)
                {
                    loadValidation(true);
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false, 2);
            }
        }

        private void revokeExpenseSheetButton_Click(object sender, EventArgs e)
        {
            if (((App)Application.Current).validationExpensesSheetListItemSelected == null)
            {
                MessageBox.Show("Has de seleccionar un full per poder rebutjar-lo", "Informació", MessageBoxButton.OK);
            }
            else
            {
                if (MessageBoxResult.OK == MessageBox.Show("Està segur que vol rebutjar el full seleccionat?", "Alerta", MessageBoxButton.OKCancel))
                {
                    InputPrompt revokedReasonInput = new InputPrompt();
                    revokedReasonInput.Completed += new EventHandler<PopUpEventArgs<string,PopUpResult>>(revokedReasonInput_Completed);
                    revokedReasonInput.Title = "Rebutjar full";
                    revokedReasonInput.Message = "Indiqui una raó per la qual es rebutja el full.";
                    //Button OKButton = new Button();
                    //OKButton.Content = "ok";
                    //OKButton.Name = "OK";
                    //Button CancelButton = new Button();
                    //CancelButton.Content = "cancel";
                    //CancelButton.Name = "Cancel";
                    //revokedReasonInput.ActionPopUpButtons = new List<Button>() { OKButton, CancelButton };
                    revokedReasonInput.Show();
                }
            }
        }

        private void revokedReasonInput_Completed(object sender, PopUpEventArgs<string, PopUpResult> e)
        {
            if (String.IsNullOrEmpty(e.Result))
            {
                MessageBox.Show("Has d'introduïr una raó per la qual rebutjes el full.", "Error", MessageBoxButton.OK);
            }
            else
            {
                try
                {
                    loading(true, 2);
                    client.revokeExpensesSheetAsync(((App)Application.Current).validationExpensesSheetListItemSelected.ExpensesSheetHandler, e.Result, ((App)Application.Current).userToken);
                }
                catch (FaultException<GestorDespesesFault> fedf)
                {
                    loading(false, 2);
                    MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
                }
                catch
                {
                    loading(false, 2);
                    MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
                }
            }
        }

        private void client_revokeExpensesSheetCompleted(object sender, GestorDespesesWS.revokeExpensesSheetCompletedEventArgs e)
        {
            try
            {
                if (e.Result)
                {
                    loadValidation(true);
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false, 2);
            }
        }
        #endregion

        private void loading(bool state,int index)
        {
            if (!state)
            {
                if (index == 0) rLoading1.Hide();
                if (index == 1) rLoading2.Hide();
                if (index == 2) rLoading3.Hide();
            }
            else
            {
                if (index == 0) rLoading1.Show();
                if (index == 1) rLoading2.Show();
                if (index == 2) rLoading3.Show();
            }
        }

        private void Pivot_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // Mostrar o ocultar ApplicationBar
            if (((PivotItem)ExpensesPivot.SelectedItem).Name == "expensesPivotItem" && expensesOpen.Visibility == Visibility.Visible)
            {
                ApplicationBar = expensesApplicationBar;
            }
            else if (((PivotItem)ExpensesPivot.SelectedItem).Name == "expensesPivotItem" && expensesValidating.Visibility == Visibility.Visible)
            {
                ApplicationBar = expensesValidatingApplicationBar;
            }
            else if (((PivotItem)ExpensesPivot.SelectedItem).Name == "historyPivotItem")
            {
                loadHistory(false);
                ApplicationBar = historyApplicationBar;
            }
            else if (((PivotItem)ExpensesPivot.SelectedItem).Name == "validationPivotItem")
            {
                loadValidation(false);
                ApplicationBar = validationApplicationBar;
            }
            else
            {
                ApplicationBar = null;
            }
        }

        private void PhoneApplicationPage_BackKeyPress(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (MessageBoxResult.OK == MessageBox.Show("Està segur que vol sortir?", "Sortir", MessageBoxButton.OKCancel))
            {
                ((App)Application.Current).reset();
                NavigationService.Navigate(new Uri("/Login.xaml", UriKind.Relative));
                e.Cancel = false;
            }
            else
            {
                e.Cancel = true;
            }
        }

        protected override void OnNavigatedFrom(System.Windows.Navigation.NavigationEventArgs e)
        {
            if (e.Content is AddEditExpensePage)
            {
                (e.Content as AddEditExpensePage).isAddMode = isAddMode;
                (e.Content as AddEditExpensePage).itemToEdit = itemToEdit;
                (e.Content as AddEditExpensePage).fromParent = true;
            }

            if (e.Content is ExpensesSheetDetailsPage)
            {
                if (((PivotItem)ExpensesPivot.SelectedItem).Name == "historyPivotItem")
                {
                    (e.Content as ExpensesSheetDetailsPage).expensesSheetId = ((HistoryExpensesSheetListItem)historyExpensesSheetList.SelectedItem).ExpensesSheetHandler;
                }
                if (((PivotItem)ExpensesPivot.SelectedItem).Name == "validationPivotItem")
                {
                    (e.Content as ExpensesSheetDetailsPage).expensesSheetId = ((ValidationExpensesSheetListItem)validationExpensesSheetList.SelectedItem).ExpensesSheetHandler;
                }
            }
            base.OnNavigatedFrom(e);
        }

        private void currentExpensesList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // disable previous selected if needed
            if (((App)Application.Current).currentExpenseListItemSelected != null) 
            {
                ((App)Application.Current).currentExpenseListItemSelected.IsSelected = false;
            }
            ((App)Application.Current).currentExpenseListItemSelected = (ExpensesListItem)currentExpensesList.SelectedItem;
            if (((App)Application.Current).currentExpenseListItemSelected != null)
            {
                ((App)Application.Current).currentExpenseListItemSelected.IsSelected = true;
            }
        }

        private void historyExpensesSheetList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // disable previous selected if needed
            if (((App)Application.Current).historyExpensesSheetListItemSelected != null) 
            {
                ((App)Application.Current).historyExpensesSheetListItemSelected.IsSelected = false;
            }
            ((App)Application.Current).historyExpensesSheetListItemSelected = (HistoryExpensesSheetListItem)historyExpensesSheetList.SelectedItem;
            if (((App)Application.Current).historyExpensesSheetListItemSelected != null)
            {
                ((App)Application.Current).historyExpensesSheetListItemSelected.IsSelected = true;
            }
        }

        private void validationExpensesSheetList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            // disable previous selected if needed
            if (((App)Application.Current).validationExpensesSheetListItemSelected != null)
            {
                ((App)Application.Current).validationExpensesSheetListItemSelected.IsSelected = false;
            }
            ((App)Application.Current).validationExpensesSheetListItemSelected = (ValidationExpensesSheetListItem)validationExpensesSheetList.SelectedItem;
            if (((App)Application.Current).validationExpensesSheetListItemSelected != null)
            {
                ((App)Application.Current).validationExpensesSheetListItemSelected.IsSelected = true;
            }
        }
    }
}