﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.ServiceModel;
using GestorDespesesWP7.GestorDespesesWS;
using System.Collections.ObjectModel;

namespace GestorDespesesWP7
{
    public partial class ExpensesSheetDetailsPage : PhoneApplicationPage
    {
        private static BasicHttpBinding binding = new BasicHttpBinding();
        private static EndpointAddress address = new EndpointAddress(((App)Application.Current).ServiceUrl);
        private GestorDespesesWSClient client = null;
        private ObservableCollection<ExpensesListItem> expensesSheetDetailsListCollection = null;

        public int expensesSheetId;

        public ExpensesSheetDetailsPage()
        {
            InitializeComponent();
            lbUserFullName.Text = ((App)Application.Current).fullName;
            this.Loaded += new RoutedEventHandler(ExpensesSheetDetailsPage_Loaded);

            expensesSheetDetailsListCollection = new ObservableCollection<ExpensesListItem>();
            expensesSheetDetailsList.ItemsSource = expensesSheetDetailsListCollection;

            client = new GestorDespesesWSClient(binding, address);

            client.getExpensesOfExpensesSheetCompleted += new EventHandler<getExpensesOfExpensesSheetCompletedEventArgs>(client_getExpensesOfExpensesSheetCompleted);
        }

        private void ExpensesSheetDetailsPage_Loaded(object sender, RoutedEventArgs e)
        { 
            //Load expenses from id
            loadExpensesSheetDetails();
        }

        private void loadExpensesSheetDetails()
        {
            try
            {
                loading(true);
                client.getExpensesOfExpensesSheetAsync(expensesSheetId,((App)Application.Current).userToken);
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                loading(false);
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                loading(false);
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
        }

        private void client_getExpensesOfExpensesSheetCompleted(object sender, GestorDespesesWS.getExpensesOfExpensesSheetCompletedEventArgs e)
        {
            try
            {
                decimal total = 0;
                expensesSheetDetailsListCollection.Clear();
                foreach (ExpenseData expense in e.Result)
                {
                    ExpensesListItem item = new ExpensesListItem();
                    item.ExpenseHandler = expense.ExpenseHandler;
                    item.Description.Text = expense.Description;
                    item.Value.Text = (expense.TypeValue) == null ? "-" : Utils.FormatDecimal(expense.TypeValue.Value);
                    item.Value2.Text = (expense.Value) == null ? "-" : Utils.FormatDecimal(expense.Value.Value);
                    item.Concept.Text = expense.TypeConcept;
                    item.Date.Text = expense.Date.ToShortDateString();
                    item.IsHolidays.Text = expense.IsOnHolidays ? "Si" : "No";
                    item.HasImage.Text = expense.HasInvoice ? "Si" : "No";
                    item.Total.Text = Utils.FormatDecimal(expense.TotalValue);
                    switch (expense.Type)
                    {
                        case ExpenseTypeType.FIXED:
                            item.Type.Text = "Fixe";
                            break;
                        case ExpenseTypeType.MULTIPLICITY:
                            item.Type.Text = "Multiplicatiu";
                            break;
                        case ExpenseTypeType.VARIABLE:
                            item.Type.Text = "Variable";
                            break;
                    }

                    expensesSheetDetailsListCollection.Add(item);
                    total += expense.TotalValue;
                }
                tbTotalValue.Text = "Total: " + Utils.FormatDecimal(total) + "€";
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false);
            }
        }

        private void loading(bool state)
        {
            if (!state)
            {
                rLoading.Hide();
            }
            else
            {
                rLoading.Show();
            }
        }
    }
}