﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.ServiceModel;
using GestorDespesesWP7.GestorDespesesWS;
using Coding4Fun.Toolkit.Controls;

namespace GestorDespesesWP7
{
    public partial class LoginPage : PhoneApplicationPage
    {
        private static BasicHttpBinding binding = new BasicHttpBinding();
        private static EndpointAddress address = new EndpointAddress(((App)Application.Current).ServiceUrl);
        private static GestorDespesesWSClient client = new GestorDespesesWSClient(binding, address);
        private static bool initialized = false;

        // Constructor
        public LoginPage()
        {
            InitializeComponent();
            loading(false);
            if (!initialized)
            {
                client.loginCompleted += new EventHandler<loginCompletedEventArgs>(client_loginCompleted);
                client.getCurrentUserCompleted += new EventHandler<getCurrentUserCompletedEventArgs>(client_getCurrentUserCompleted);
                initialized = true;
            }
        }

        private void hbtRecover_Click(object sender, RoutedEventArgs e)
        {
            NavigationService.Navigate(new Uri("/Recover.xaml", UriKind.Relative));
        }

        private void hbtAbout_Click(object sender, RoutedEventArgs e)
        {
            AboutPrompt about = new AboutPrompt();
            about.Title = "Gestor Despeses UOC";
            about.Footer = @"http://www.uoc.edu/";
            about.VersionNumber = "Version 1.0";
            about.Body = new TextBlock { Text = "Treball Final de Carrera", TextWrapping = TextWrapping.Wrap };
            about.Show("Xavier Miñana Garcia", null, "xminana@uoc.edu", null);
        }

        private void tbLogin_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                client.loginAsync(tbUsuari.Text, tbPassword.Password);
                loading(true);
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                loading(false);
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                loading(false);
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
        }

        private void client_loginCompleted(object sender, GestorDespesesWS.loginCompletedEventArgs e)
        {
            try
            {
                ((App)Application.Current).userToken = e.Result;
                client.getCurrentUserAsync(((App)Application.Current).userToken);
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                loading(false);
                ((App)Application.Current).reset();
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                loading(false);
                ((App)Application.Current).reset();
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
        }

        private void client_getCurrentUserCompleted(object sender, GestorDespesesWS.getCurrentUserCompletedEventArgs e)
        {
            try
            {
                UserData user = e.Result;
                switch (user.Type) { 
                    case UserType.WORKER:
                        //Fill global variables with user information
                        ((App)Application.Current).userId = user.UserHandler;
                        ((App)Application.Current).userType = user.Type;
                        ((App)Application.Current).fullName = user.FullName;

                        NavigationService.Navigate(new Uri("/ExpensesPivotPage.xaml", UriKind.Relative));
                        break;
                    case UserType.ADMINISTRATOR:
                    default:
                        MessageBox.Show("Per administrar el servei accedeixi mitjançant l'aplicacio web.", "Error", MessageBoxButton.OK);
                        loading(false);
                        ((App)Application.Current).reset(); 
                        break;
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                loading(false);
                ((App)Application.Current).reset(); 
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                loading(false);
                ((App)Application.Current).reset();
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
        }

        private void loading(bool state)
        {
            if (!state)
            {
                rLoading.Hide();
            }
            else
            {
                rLoading.Show();
            }
        }

        private void PhoneApplicationPage_BackKeyPress(object sender, System.ComponentModel.CancelEventArgs e)
        {
            if (NavigationService.CanGoBack)
            {
                while (NavigationService.RemoveBackEntry() != null)
                {
                    NavigationService.RemoveBackEntry();
                }
            }
        }
    }
}