﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.ServiceModel;
using GestorDespesesWP7.GestorDespesesWS;

namespace GestorDespesesWP7
{
    public partial class Recover : PhoneApplicationPage
    {
        private static BasicHttpBinding binding = new BasicHttpBinding();
        private static EndpointAddress address = new EndpointAddress(((App)Application.Current).ServiceUrl);
        private static GestorDespesesWSClient client = new GestorDespesesWSClient(binding, address);
        private static bool initialized = false;

        public Recover()
        {
            InitializeComponent();
            loading(false);
            if (!initialized)
            {
                client.generateValidationCodeCompleted += new EventHandler<generateValidationCodeCompletedEventArgs>(client_generateValidationCodeCompleted);
                client.changePasswordCompleted += new EventHandler<changePasswordCompletedEventArgs>(client_changePasswordCompleted);
                initialized = true;
            }
        }

        #region Validacio
        
        private void btValidar_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                loading(true);
                client.generateValidationCodeAsync(tbUserOrEmail.Text);
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                loading(false);
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                loading(false);
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
        }

        private void client_generateValidationCodeCompleted(object sender, GestorDespesesWS.generateValidationCodeCompletedEventArgs e)
        {
            try
            {
                if (e.Result)
                {
                    MessageBox.Show("Codi de validació enviat a la teva bústia.", "Informació", MessageBoxButton.OK);
                }
                else
                {
                    // Mai arribarem aqui perque el WS en cas d'error llença una excepció
                    throw new Exception("Error de validació");
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false);
            }
        }
        
        #endregion

        #region Cambi de credencials
        
        private void btRecuperar_Click(object sender, RoutedEventArgs e)
        {
            if (tbPassword.Password.Equals(tbConfirm.Password))
            {

                try
                {
                    loading(true);
                    client.changePasswordAsync(tbPassword.Password, tbCodi.Text);
                }
                catch (FaultException<GestorDespesesFault> fedf)
                {
                    loading(false);
                    MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
                }
                catch
                {
                    loading(false);
                    MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
                }
            }
            else 
            {
                MessageBox.Show("La confirmació de la contrasenya no és correcte.", "Error", MessageBoxButton.OK);
            }
        }

        private void client_changePasswordCompleted(object sender, GestorDespesesWS.changePasswordCompletedEventArgs e)
        {
            try
            {
                if (e.Result)
                {
                    if (MessageBox.Show("Credencials modificades correctament.", "Informació", MessageBoxButton.OK) == MessageBoxResult.OK) 
                    {
                        //Tornem a la pagina de login.
                        NavigationService.GoBack();
                    }
                }
                else
                {
                    // Mai arribarem aqui perque el WS en cas d'error llença una excepció
                    throw new Exception("Error al recuperar credencials");
                }
            }
            catch (FaultException<GestorDespesesFault> fedf)
            {
                MessageBox.Show(fedf.Detail.Message, "Error", MessageBoxButton.OK);
            }
            catch
            {
                MessageBox.Show("Error desconegut", "Error", MessageBoxButton.OK);
            }
            finally
            {
                loading(false);
            }
        }

        #endregion

        private void loading(bool state)
        {
            if (!state)
            {
                rLoading.Hide();
            }
            else
            {
                rLoading.Show();
            }
        }
    }
} 